#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <math.h>
#include <float.h>

#define PI        3.141592653589793238462643383279502884197L

#define MAX(a, b) ((a) > (b) ? (a) : (b))
#define MIN(a, b) ((a) < (b) ? (a) : (b))

double Kepler(double x, double e)
{
    int cnt = 0;
    bool neg_e = false, adjust = false;
    double x1, x2, s, c, d1, d2, d, a, b, v;
    if(fabs(e) > 1.0)
        return 0.0L;
    if(sin(x) == 0.0)
        return x;
    if(e < 0)
    {
        e = -e;
        x -= PI;
        neg_e = true;
    }
    a = x - e;
    b = x + e;
    if(sin(x / 2) > 0)
        x1 = MAX(a, (long double)(lround((x - PI) / (2 * PI))) * (2 * PI) + PI);
    else
        x1 = MIN(b, (long double)(lround((x - PI) / (2 * PI))) * (2 * PI) + PI);
    sincos(x1, &s, &c);
    v = (1 - e * c);
    d1 = (x1 - e * s - x) / v;
    do
    {
        x2 = x1;
        d2 = d1;
        x1 -= d1;

        sincos(x1 , &s, &c);
        d1 = (x1 - e * s - x);

        if(d1 == 0.0L)
            break;
        if((d1 < 0) == (d2 > 0))
        {
            ++cnt;
            if(cnt > 1)
            {
                adjust = true;
                break;
            }
        }
        if(cnt == 0)
            v = (1 - e * c);
        d1 /= v;
    }while(fabs((x1 - x2) / x2) > 2 * DBL_EPSILON);
    if(adjust)
    {
        do
        {
            c = (x1 + x2) / 2;
            d = c - e * sin(c) - x;
            if(d == 0)
            {
                x1 = c;
                break;
            }
            if((d < 0) == (d1 > 0))
            {
                x2 = c;
                d2 = d;
            }
            else
            {
                x1 = c;
                d1 = d;
            }
        }while(fabs((x1 - x2) / x2) > DBL_EPSILON);
        x1 = c;
    }
    if(neg_e) return x1 + PI;
    else return x1;
}


long double KeplerL(long double x, long double e)
{
    int cnt = 0;    
    bool neg_e = false, adjust = false;
    long double x1, x2, s, c, d1, d2, d, a, b, v;
    if(fabsl(e) > 1.0L)
        return 0.0L;
    if(sinl(x) == 0.0L)
        return x;
    if(e < 0)
    {
        e = -e;
        x -= PI;
        neg_e = true;
    }
    a = x - e;
    b = x + e;
    if(sinl(x / 2) > 0)
        x1 = MAX(a, (long double)(lroundl((x - PI) / (2 * PI))) * (2 * PI) + PI);
    else
        x1 = MIN(b, (long double)(lroundl((x - PI) / (2 * PI))) * (2 * PI) + PI);
    sincosl(x1, &s, &c);
    v = (1 - e * c);
    d1 = (x1 - e * s - x) / v;
    do
    {
        x2 = x1;
        d2 = d1;
        x1 -= d1;

        sincosl(x1 , &s, &c);
        d1 = (x1 - e * s - x);

        if(d1 == 0.0L)
            break;
        if((d1 < 0) == (d2 > 0))
        {
            ++cnt;
            if(cnt > 1)
            {
                adjust = true;
                break;
            }
        }
        if(cnt == 0)
            v = (1 - e * c);
        d1 /= v;        
    }while(fabsl((x1 - x2) / x2) > 2 * LDBL_EPSILON);
    if(adjust)
    {
        do
        {            
            c = (x1 + x2) / 2;
            d = c - e * sinl(c) - x;
            if(d == 0)
            {
                x1 = c;
                break;
            }
            if((d < 0) == (d1 > 0))
            {
                x2 = c;
                d2 = d;
            }
            else
            {
                x1 = c;
                d1 = d;
            }
        }while(fabsl((x1 - x2) / x2) > LDBL_EPSILON);
        x1 = c;
    }    
    if(neg_e) return x1 + PI;
    else return x1;
}

long double random01()
{
    int s;
    unsigned long long r = (unsigned)(rand());    
    r = r * RAND_MAX + (unsigned)(rand()) ^ (unsigned)(rand());    
    r = r * RAND_MAX + (unsigned)(rand()) ^ (unsigned)(rand());
    r = r * RAND_MAX + (unsigned)(rand()) ^ (unsigned)(rand());
    s = rand() ^ rand();
    r = (r << 4) | (s & 0xf);    
    return ((long double)r) / 0x100000000 / 0x100000000;    
}


const int N = 1000; // relation of the given precision to the machine precision

double KeplerDanby(double x, double e, int *cnt)
{
    double s, dnm, E0, E1, ME;
    *cnt = 0;
    E1 = x + 0.85 * e;
    do
    {
        ++(*cnt);
        E0 = E1;
        s = sin(E0);
        ME = x + e * s;
        s = sin(ME);
        dnm = E0 - 2 * ME + x + e * s;
        if(dnm == 0)
            break;
        E1 = E0 - (ME - E0) * (ME - E0) / dnm;
    }while(fabs((E0 - E1) / E1) >= N * DBL_EPSILON && (*cnt) <= 500);
    return E1;        
}

 

int main(int argc, char** argv) 
{
    
    int i = 0, c;
    long double m, dl = 0, ci = 0, e;
    double kd;
    srand(time(NULL));
    while(i < 499)
    {
        m = random01() * PI;
        e = 0.6L + random01() * 0.39L;  // finding problem M and e value
        kd = KeplerDanby(m, e, &i);                          
    }
    printf("\n M = %+-#.20Lg \n", m);
    printf("\n e = %+-#.20Lg \n", e);
    printf("\n 64 bit Danby Kepler equation solution for these M, e is  %+-#.16g \n", kd);
    printf("\n 64 bit Kepler equation solution for these M, e at machine precision is %+-#.16g \n", Kepler(m,  e));
    printf("\n 80 bit Kepler equation solution for these M, e at machine precision is %+-#.20Lg \n", KeplerL(m,  e));
    fflush(0);
    getch();
    return 0;
}