#include <math.h>
#include <float.h>
#include <stdbool.h>
#include <quadmath.h>

__float128 KeplerQ(__float128 x, __float128 e)
{    
    int cnt = 0;    
    bool neg_e = false, adjust = false;
    __float128 x1, x2, s, c, d1, d2, d, a, b, v;
    if(fabsq(e) > 1.0L)
        return 0.0L;    
    if(sinq(x) == 0.0L)
        return x;    
    if(e < 0)
    {
        e = -e;
        x -= PI;        
        neg_e = true;
    }
    a = x - e;
    b = x + e;
    if(sinq(x / 2) > 0)    
        x1 = MAX(a, (__float128)(lroundq((x - PI) / (2 * PI))) * (2 * PI) + PI);
    else
        x1 = MIN(b, (__float128)(lroundq((x - PI) / (2 * PI))) * (2 * PI) + PI);
    sincosq(x1, &s, &c);
    v = (1 - e * c);
    d1 = (x1 - e * s - x) / v;    
    do
    {           
        x2 = x1;
        d2 = d1;        
        x1 -= d1;        
        sincosq(x1 , &s, &c);
        d1 = (x1 - e * s - x);
        if(d1 == 0.0L)
            break;        
        if((d1 < 0) == (d2 > 0))
        {
            ++cnt;
            if(cnt > 1)
            {
                adjust = true;
                break;
            }
        }        
        if(cnt == 0)
            v = (1 - e * c);
        d1 /= v;
    }while(fabsq((x1 - x2) / x2) > 2 * FLT128_EPSILON);        
    if(adjust)
    {
        do
        {              
            c = (x1 + x2) / 2;
            d = c - e * sinq(c) - x;
            if(d == 0)
            {
                x1 = c;
                break;
            }
            if((d < 0) == (d1 > 0))
            {
                x2 = c;
                d2 = d;
            }
            else
            {
                x1 = c;
                d1 = d;
            }
        }while(fabsq((x1 - x2) / x2) > FLT128_EPSILON);
        x1 = c;
    }
    if(neg_e) return x1 + PI;
    else return x1;
}